
//////////////////////////////////////////////////
//  SecureBridge Components
//  Copyright  2023-2024 Devart. All right reserved.
//  HTTP Server Demo
//////////////////////////////////////////////////

unit HttpServerMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, Spin, FileCtrl,
  ScVioSocket, ScBridge, ScCertificateExts, ScUtils, ScAlgorithmSupport,
  ScCLRClasses, ScTCPServer, ScSSLServer, ScHttpServer, ScHttp,
  ScDualListEditor, ScCertificateEditor;

type
  THttpServerForm = class(TForm)
    ScFileStorage: TScFileStorage;
    ScHttpServer: TScHttpServer;
    pnMain: TPanel;
    lblAddress: TLabel;
    cbListenAddress: TComboBox;
    lblPort: TLabel;
    sePort: TSpinEdit;
    cbIsSecure: TCheckBox;
    lblPath: TLabel;
    edStoragePath: TEdit;
    sbStoragePath: TSpeedButton;
    lblCertificateChain: TLabel;
    edCertificateChain: TEdit;
    sbCertificateChain: TSpeedButton;
    lblCACertificate: TLabel;
    cbCACertificate: TComboBox;
    lblCipherSuites: TLabel;
    edCipherSuites: TEdit;
    sbCipherSuites: TSpeedButton;
    lblTimeout: TLabel;
    seTimeout: TSpinEdit;
    ibiMaxAccepted: TLabel;
    seMaxAccepted: TSpinEdit;
    shState: TShape;
    btStartStop: TButton;
    btTest: TButton;
    procedure FormCreate(Sender: TObject);
    procedure edStoragePathChange(Sender: TObject);
    procedure sbStoragePathClick(Sender: TObject);
    procedure sbCipherSuitesClick(Sender: TObject);
    procedure sbCertificateChainClick(Sender: TObject);
    procedure btStartStopClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cbCACertificateDropDown(Sender: TObject);
    procedure ScHttpServerProcessRequest(Sender: TObject;
      RequestInfo: TScHttpRequestInfo; ResponseInfo: TScHttpResponseInfo);
    procedure btTestClick(Sender: TObject);
  private
    FDualListEditorForm: TScDualListEditorForm;
    FCertificateEditorForm: TScCertificateEditorForm;

    procedure DoAfterDeactivate(Sender: TObject);
    procedure DoCipherSuitesChanged(Sender: TObject);
    procedure DoCertificateChainChanged(Sender: TObject);
  public
    { Public declarations }
  end;

var
  HttpServerForm: THttpServerForm;

implementation

{$R *.dfm}

const
  SAllInterfaces = '0.0.0.0 (All interfaces)';

procedure THttpServerForm.FormCreate(Sender: TObject);
begin
  GetIPInterfaces(cbListenAddress.Items);
  cbListenAddress.Items.Insert(0, SAllInterfaces);
  cbListenAddress.ItemIndex := 0;

  ScHttpServer.SSLOptions.CipherSuites.OnChanged := DoCipherSuitesChanged;
  edCipherSuites.Text := ScHttpServer.SSLOptions.CipherSuites.AsString;

  ScHttpServer.AfterDeactivate := DoAfterDeactivate;
  ScFileStorage.Certificates.CertificateByName('ca').Key.Assign(ScFileStorage.Keys.KeyByName('ca'));
  ScFileStorage.Certificates.CertificateByName('server').Key.Assign(ScFileStorage.Keys.KeyByName('server'));
  edCertificateChain.Text := 'server;ca';
end;

procedure THttpServerForm.sbStoragePathClick(Sender: TObject);
var
  Dir: string;
begin
  Dir := edStoragePath.Text;
  if SelectDirectory(Dir, [sdAllowCreate, sdPerformCreate, sdPrompt], 0) then
    edStoragePath.Text := Dir;
end;

procedure THttpServerForm.edStoragePathChange(Sender: TObject);
begin
  ScFileStorage.Path := ExpandFileName(edStoragePath.Text);
end;

procedure THttpServerForm.sbCertificateChainClick(Sender: TObject);
begin
  if not Assigned(FCertificateEditorForm) then begin
    FCertificateEditorForm := TScCertificateEditorForm.Create(Self);
    FCertificateEditorForm.OnChanged := DoCertificateChainChanged;
  end;

  FCertificateEditorForm.Storage := ScFileStorage;
  FCertificateEditorForm.ShowModal;
end;

procedure THttpServerForm.DoCertificateChainChanged(Sender: TObject);
begin
  if Assigned(FCertificateEditorForm) then
    edCertificateChain.Text := FCertificateEditorForm.CertificateList;
end;

procedure THttpServerForm.cbCACertificateDropDown(Sender: TObject);
var
  i: integer;
begin
  cbCACertificate.Items.BeginUpdate;
  try
    cbCACertificate.Items.Clear;
    for i := 0 to ScFileStorage.Certificates.Count - 1 do
      cbCACertificate.Items.Add(ScFileStorage.Certificates[i].CertName);
  finally
    cbCACertificate.Items.EndUpdate;
  end;
end;

procedure THttpServerForm.sbCipherSuitesClick(Sender: TObject);
begin
  if not Assigned(FDualListEditorForm) then
    FDualListEditorForm := TScDualListEditorForm.Create(Self);

  FDualListEditorForm.SSLCipherSuites := ScHttpServer.SSLOptions.CipherSuites;
  FDualListEditorForm.ShowModal;
end;

procedure THttpServerForm.DoCipherSuitesChanged(Sender: TObject);
begin
  edCipherSuites.Text := ScHttpServer.SSLOptions.CipherSuites.AsString;
end;

procedure THttpServerForm.btStartStopClick(Sender: TObject);
begin
  if ScHttpServer.Active then
    ScHttpServer.Active := False
  else begin
    if cbListenAddress.Text = SAllInterfaces then
      ScHttpServer.BindAddress := '0.0.0.0'
    else
      ScHttpServer.BindAddress := cbListenAddress.Text;

    ScHttpServer.Port := sePort.Value;
    ScHttpServer.IsSecure := cbIsSecure.Checked;
    ScHttpServer.CertNameChain := edCertificateChain.Text;
    ScHttpServer.CACertName := cbCACertificate.Text;
    ScHttpServer.SSLOptions.CipherSuites.AsString := edCipherSuites.Text;
    ScHttpServer.Timeout := seTimeout.Value;
    ScHttpServer.Options.MaxAcceptedConnections := seMaxAccepted.Value;
    ScHttpServer.Active := True;

    btStartStop.Caption := 'Stop server';
    shState.Brush.Color := clGreen;
  end;
end;

procedure THttpServerForm.DoAfterDeactivate(Sender: TObject);
begin
  btStartStop.Caption := 'Start server';
  shState.Brush.Color := clRed;
end;

procedure THttpServerForm.ScHttpServerProcessRequest(Sender: TObject;
  RequestInfo: TScHttpRequestInfo; ResponseInfo: TScHttpResponseInfo);
var
  FileStream: TFileStream;
  fn: string;
begin
  if RequestInfo.HttpMethod = rmGET then begin
    if RequestInfo.RawURI = '/' then begin
      // Send the main page
      ResponseInfo.SendChunked := False;
      ResponseInfo.OutputText :=
        '<html>' +
        '<head><title>Test page</title></head>' +
        '<body>This is the main page of the HTTP server test.</body>' +
        '</html>';
      ResponseInfo.StatusCode := scOK;
    end
    else begin
      // Read data from the file and send it to the client
      try
        fn := StringReplace(RequestInfo.RawURI, '/', '\', [rfReplaceAll]);
        ResponseInfo.OutputStream := TFileStream.Create('.' + fn, fmOpenRead or fmShareDenyWrite);
        ResponseInfo.IsOutputStreamOwner := True;
        ResponseInfo.StatusCode := scOK;
      except
        ResponseInfo.StatusCode := scNotFound;
      end;
    end;
  end
  else
  if RequestInfo.HttpMethod = rmPOST then begin
    if RequestInfo.RawURI = '/' then
      ResponseInfo.StatusCode := scNotFound
    else begin
      // Save the received data to a file
      fn := StringReplace(RequestInfo.RawURI, '/', '\', [rfReplaceAll]);
      FileStream := TFileStream.Create('.' + fn, fmCreate);
      FileStream.CopyFrom(RequestInfo.InputStream, RequestInfo.InputStream.Size);

      ResponseInfo.SendChunked := False;
      ResponseInfo.ContentType := 'text/plain';
      ResponseInfo.OutputText := 'Data was post successfully';
      ResponseInfo.StatusCode := scOK;
    end;
  end
  else
    ResponseInfo.StatusCode := scForbidden;
end;

procedure THttpServerForm.btTestClick(Sender: TObject);
const
  TestString = '<html><head><title>Hello page</title></head><body>Hello!</body></html>';
  TestsMethod = rmGET; // rmPOST;
var
  TestRequest: TScHttpWebRequest;
  TestResponse: TScHttpWebResponse;
  RespStr: string;
begin
  TestResponse := nil;
  TestRequest := TScHttpWebRequest.Create('https://127.0.0.1/readme.html');
  try
    TestRequest.SSLOptions.TrustServerCertificate := True;
    TestRequest.SSLOptions.CipherSuites.AsString := edCipherSuites.Text;

    TestRequest.Method := TestsMethod;

    if TestRequest.Method = rmPOST then begin
      TestRequest.WriteBuffer(Encoding.Default.GetBytes(TestString));
      TestRequest.ContentLength := Length(TestString);
    end;

    TestResponse := TestRequest.GetResponse;
    RespStr := TestResponse.ReadAsString;
    ShowMessage(RespStr);
  finally
    TestRequest.Free;
    TestResponse.Free;
  end;
end;

procedure THttpServerForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  ScHttpServer.Active := False;
end;

end.
