
//////////////////////////////////////////////////
//  SecureBridge Components
//  Copyright  2023-2024 Devart. All right reserved.
//  Certificate Editor
//////////////////////////////////////////////////

unit ScCertificateEditor;

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Grids, Buttons, ExtCtrls, StdCtrls,
{$IFDEF FPC}
  LResources, LCLType,
{$ENDIF}
  ScTypes, ScUtils, ScBridge, ScCertificateExts, ScAlgorithmSupport;

type
  TScCertificateEditorForm = class(TForm)
    pnBottom: TPanel;
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    imCorner: TImage;
    pnCertificateList: TPanel;
    btAdd: TButton;
    btDelete: TButton;
    lbCertificateList: TListBox;
    pnCertificateInfo: TPanel;
    lblCertificateName: TLabel;
    lblProperties: TLabel;
    lblKey: TLabel;
    lblFingerprint: TLabel;
    edCertificateName: TEdit;
    mmProperties: TMemo;
    edKey: TEdit;
    edFingerprint: TEdit;
    btImportPrivateKey: TButton;
    cbIsPrivate: TCheckBox;
    Splitter1: TSplitter;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure btAddClick(Sender: TObject);
    procedure btDeleteClick(Sender: TObject);
    procedure btImportPrivateKeyClick(Sender: TObject);
    procedure lbCertificateListClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
  private
    FStorage: TScFileStorage;
    FCertificateList: string;
    FOnChanged: TNotifyEvent;

    procedure UpdateCertificateList;
    procedure UpdateCertificateInfo(Cert: TScCertificate);
  public
    property Storage: TScFileStorage read FStorage write FStorage;
    property CertificateList: string read FCertificateList;
    property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  end;

implementation

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

procedure TScCertificateEditorForm.UpdateCertificateList;
var
  Cert: TScCertificate;
  i: integer;
begin
  lbCertificateList.Items.BeginUpdate;
  try
    lbCertificateList.Items.Clear;
    for i := 0 to FStorage.Certificates.Count - 1 do begin
      Cert := FStorage.Certificates[i];
      lbCertificateList.Items.AddObject(Cert.CertName, Cert);
    end;

    btDelete.Enabled := FStorage.Certificates.Count > 0;

    if FStorage.Certificates.Count > 0 then begin
      lbCertificateList.ItemIndex := 0;
      UpdateCertificateInfo(TScCertificate(lbCertificateList.Items.Objects[0]));
    end
    else
      UpdateCertificateInfo(nil);
  finally
    lbCertificateList.Items.EndUpdate;
  end;
end;

procedure TScCertificateEditorForm.UpdateCertificateInfo(Cert: TScCertificate);
var
  Extension: TScCertificateExtension;
  fp: string;
  i: integer;
begin
  if (Cert <> nil) and Cert.Ready then begin
    pnCertificateInfo.Enabled := True;

    edCertificateName.Text := Cert.Subject;
    mmProperties.Lines.Clear;
    mmProperties.Lines.Add('Version: v' + IntToStr(Cert.Version));
    mmProperties.Lines.Add('Serial number: ' + Cert.SerialNumber);
    mmProperties.Lines.Add('Signature algorithm: ' + Cert.SignatureAlgorithm.Algorithm.FriendlyName);
    mmProperties.Lines.Add('Valid from: ' + DateTimeToStr(Cert.NotBefore));
    mmProperties.Lines.Add('Valid to: ' + DateTimeToStr(Cert.NotAfter));
    mmProperties.Lines.Add('Issuer: ' + Cert.IssuerName.ToString);
    mmProperties.Lines.Add('Subject: ' + Cert.SubjectName.ToString);
    Cert.GetFingerprint(haSHA1, fp);
    mmProperties.Lines.Add('Thumbprint with SHA1: ' + fp);

    for i := 0 to Cert.Extensions.Count - 1 do begin
      if i = 0 then
        mmProperties.Lines.Add('Extensions:');

      Extension := Cert.Extensions.Extensions[i];
      mmProperties.Lines.Add('    ' + Extension.Oid.FriendlyName + ':');
      mmProperties.Lines.Add('        ' + Extension.ToString);
    end;

    mmProperties.SelStart := 0;
    mmProperties.SelLength := 0;

    edKey.Text := CipherFactory.PublicKeyAlgorithmToName(Cert.Key.Algorithm) + ':' + IntToStr(Cert.Key.BitCount);
    cbIsPrivate.Checked := Cert.Key.IsPrivate;
    Cert.Key.GetFingerprint(haSHA1, fp);
    edFingerprint.Text := fp;

    btImportPrivateKey.Enabled := not cbIsPrivate.Checked;
  end
  else begin
    pnCertificateInfo.Enabled := False;

    edCertificateName.Text := '';
    mmProperties.Lines.Clear;
    edKey.Text := '';
    cbIsPrivate.Checked := False;
    edFingerprint.Text := '';
    btImportPrivateKey.Enabled := False;
  end;
end;

procedure TScCertificateEditorForm.btAddClick(Sender: TObject);
var
  FileOpen: TOpenDialog;
  NewCertname: string;
  Cert: TScCertificate;
  i: integer;
begin
  i := 0;
  repeat
    Inc(i);
    NewCertname := 'cert' + IntToStr(i);
  until FStorage.Certificates.FindCertificate(NewCertname) = nil;

  FileOpen := TOpenDialog.Create(Self);
  try
    FileOpen.Title := 'Import certificate';
    FileOpen.Filename := '';
    FileOpen.Filter := 'All formats (*.pem;*.crt;*.cer)|*.pem;*.crt;*.cer|PEM format (*.pem;*.crt)|*.pem;*.crt|DER format (*.cer)|*.cer|All files (*.*)|*.*';
    FileOpen.Options := FileOpen.Options + [ofPathMustExist, ofFileMustExist];

    if FileOpen.Execute then begin
      Cert := TScCertificate.Create(nil);
      try
        Cert.CertName := NewCertname;
        Cert.ImportFrom(FileOpen.Filename);
        FStorage.Certificates.Add(Cert);
      except
        Cert.Free;
        raise;
      end;

      UpdateCertificateList;
    end;
  finally
    FileOpen.Free;
  end;
end;

procedure TScCertificateEditorForm.btDeleteClick(Sender: TObject);
var
  Cert: TScCertificate;
begin
  if (lbCertificateList.ItemIndex < 0) or (lbCertificateList.Items.Objects[lbCertificateList.ItemIndex] = nil) then
    Exit;

  try
    Cert := TScCertificate(lbCertificateList.Items.Objects[lbCertificateList.ItemIndex]);
    FStorage.Certificates.Remove(Cert);
  finally
    UpdateCertificateList;
  end;
end;

procedure TScCertificateEditorForm.btImportPrivateKeyClick(Sender: TObject);
var
  FileOpen: TOpenDialog;
  Cert: TScCertificate;
  Comment: string;
begin
  if (lbCertificateList.ItemIndex < 0) or (lbCertificateList.Items.Objects[lbCertificateList.ItemIndex] = nil) then
    Exit;

  FileOpen := TOpenDialog.Create(Self);
  try
    FileOpen.Title := 'Import key';
    FileOpen.Filename := '';
    FileOpen.Filter := 'All formats |*.ssl;*.pem;*.ietf;*.pub;*.ietfpub|OpenSSL format (*.ssl)|*.ssl|PKCS8 format (*.pem)|*.pem|IETF format (*.ietf)|*.ietf|Public key (*.pub)|*.pub|Public IETF key (*.ietfpub)|*.ietfpub|PuTTY key (*.ppk)|*.ppk|All files (*.*)|*.*';
    FileOpen.Options := FileOpen.Options + [ofPathMustExist, ofFileMustExist];

    if FileOpen.Execute then begin
      Cert := TScCertificate(lbCertificateList.Items.Objects[lbCertificateList.ItemIndex]);
      Cert.Key.ImportFrom(FileOpen.Filename, '', Comment);
      UpdateCertificateInfo(Cert);
    end;
  finally
    FileOpen.Free;
  end;
end;

procedure TScCertificateEditorForm.lbCertificateListClick(Sender: TObject);
var
  Cert: TScCertificate;
begin
  if lbCertificateList.ItemIndex < 0 then
    Exit;

  Cert := TScCertificate(lbCertificateList.Items.Objects[lbCertificateList.ItemIndex]);
  UpdateCertificateInfo(Cert);
end;

procedure TScCertificateEditorForm.FormShow(Sender: TObject);
begin
  UpdateCertificateInfo(nil);
  UpdateCertificateList;
end;

procedure TScCertificateEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i: integer;
begin
  if ModalResult <> mrOk then
    Exit;

  FCertificateList := '';
  for i := 0 to lbCertificateList.Items.Count - 1 do
    if lbCertificateList.Selected[i] then begin
      if FCertificateList <> '' then
        FCertificateList := FCertificateList + '; ';

      FCertificateList := FCertificateList + lbCertificateList.Items[i];
    end;

  if Assigned(OnChanged) then
    OnChanged(Self);
end;

procedure TScCertificateEditorForm.FormResize(Sender: TObject);
begin
  pnCertificateList.Width := (ClientWidth div 5) * 2;
end;

procedure TScCertificateEditorForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then begin
    Close;
    Key := #0;
  end;
end;

end.
