
//////////////////////////////////////////////////
//  SecureBridge Components
//  Copyright  2023-2024 Devart. All right reserved.
//  Dual List Editor
//////////////////////////////////////////////////

unit ScDualListEditor;

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Grids, Buttons, ExtCtrls, StdCtrls,
{$IFDEF FPC}
  LResources, LCLType,
{$ENDIF}
  ScTypes, ScUtils, ScSSLTypes;

type
  TScDualListEditorForm = class(TForm)
    pnBottom: TPanel;
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    imCorner: TImage;
    pnMain: TPanel;
    pnSource: TPanel;
    lblSource: TLabel;
    lbSource: TListBox;
    pnCenter: TPanel;
    btnInclude: TSpeedButton;
    btnIncludeAll: TSpeedButton;
    btnExclude: TSpeedButton;
    btnExcludeAll: TSpeedButton;
    pnDest: TPanel;
    lblDest: TLabel;
    lbDest: TListBox;
    pnRight: TPanel;
    btnUp: TSpeedButton;
    btnDown: TSpeedButton;
    procedure btnIncludeClick(Sender: TObject);
    procedure btnExcludeClick(Sender: TObject);
    procedure btnIncludeAllClick(Sender: TObject);
    procedure btnExcludeAllClick(Sender: TObject);
    procedure lbDestDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lbDestDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lbSourceDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lbSourceDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure lbDestKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  protected
    FModified: boolean;
    FSSLCipherSuites: TScSSLCipherSuites;

    procedure MoveSelected(Source, Dest: TListBox; Index: integer; SelectNext: boolean);
    procedure MoveAll(Source, Dest: TListBox; Index: integer);
    procedure Move(List: TListBox; FromIndex, ToIndex: integer);
    function GetFirstSelection(List: TCustomListBox): integer;
    procedure SetButtons;

    procedure GetSrcListItems(Items: TStrings);
    procedure GetDstListItems(Items: TStrings);
  public
    property SSLCipherSuites: TScSSLCipherSuites read FSSLCipherSuites write FSSLCipherSuites;
  end;

implementation

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  ScFunctions, ScCipherSuites;

procedure TScDualListEditorForm.GetSrcListItems(Items: TStrings);
var
  c: TScSSLCipherAlgorithm;
begin
  Assert(FSSLCipherSuites <> nil);

  Items.Clear;
  for c := Low(TScSSLCipherAlgorithm) to High(TScSSLCipherAlgorithm) do
    Items.Add(TCipherSuites.CipherAlgorithmToName(c));
end;

procedure TScDualListEditorForm.GetDstListItems(Items: TStrings);
var
  i: integer;
begin
  Assert(FSSLCipherSuites <> nil);

  Items.Clear;
  for i := 0 to FSSLCipherSuites.Count - 1 do
    Items.Add(FSSLCipherSuites[i].AsString);
end;

procedure TScDualListEditorForm.btnIncludeClick(Sender: TObject);
begin
  MoveSelected(lbSource, lbDest, lbDest.Items.Count, True);
end;

procedure TScDualListEditorForm.btnExcludeClick(Sender: TObject);
begin
  MoveSelected(lbDest, lbSource, lbSource.Items.Count, True);
end;

procedure TScDualListEditorForm.btnIncludeAllClick(Sender: TObject);
begin
  MoveAll(lbSource, lbDest, lbDest.Items.Count);
end;

procedure TScDualListEditorForm.btnExcludeAllClick(Sender: TObject);
begin
  MoveAll(lbDest, lbSource, lbSource.Items.Count);
end;

procedure TScDualListEditorForm.MoveSelected(Source, Dest: TListBox; Index: integer;
  SelectNext: boolean);
var
  i, FirstSel: integer;
begin
  for i := 0 to Dest.Items.Count - 1 do
    Dest.Selected[i] := False;

  FirstSel := -1;
  for i := Source.Items.Count - 1 downto 0 do
    if Source.Selected[i] then begin
      if Dest.Sorted then
        Dest.Items.AddObject(Source.Items[i], Source.Items.Objects[i])
      else begin
        Dest.Items.InsertObject(Index, Source.Items[i], Source.Items.Objects[i]);
        Dest.Selected[Index] := True;
      end;
      Source.Items.Delete(i);
      FirstSel := i;
    end;

  if FirstSel >= 0 then begin
    if SelectNext then
      if Source.Items.Count > FirstSel then
        Source.Selected[FirstSel] := True
      else
      if Source.Items.Count > 0 then
        Source.Selected[Source.Items.Count - 1] := True;

    FModified := True;
    SetButtons;
  end;
end;

procedure TScDualListEditorForm.MoveAll(Source, Dest: TListBox; Index: integer);
var
  i: integer;
begin
  for i := 0 to Dest.Items.Count - 1 do
    Dest.Selected[i] := False;

  for i := Source.Items.Count - 1 downto 0 do begin
    if Dest.Sorted then
      Dest.Items.AddObject(Source.Items[i], Source.Items.Objects[i])
    else begin
      Dest.Items.InsertObject(Index, Source.Items[i], Source.Items.Objects[i]);
      Dest.Selected[Index] := True;
    end;
  end;
  
  Source.Items.Clear;

  FModified := True;
  SetButtons;
end;

procedure TScDualListEditorForm.SetButtons;
begin
  btnInclude.Enabled := lbSource.SelCount > 0;
  btnIncludeAll.Enabled := lbSource.Items.Count <> 0;
  btnExclude.Enabled := lbDest.SelCount > 0;
  btnExcludeAll.Enabled := lbDest.Items.Count <> 0;
  btnUp.Enabled := (lbDest.SelCount = 1) and (GetFirstSelection(lbDest) > 0);
  btnDown.Enabled := (lbDest.SelCount = 1) and (GetFirstSelection(lbDest) < lbDest.Items.Count - 1);
end;

function TScDualListEditorForm.GetFirstSelection(List: TCustomListBox): integer;
var
  i: integer;
begin
  for i := 0 to List.Items.Count - 1 do
    if List.Selected[i] then begin
      Result := i;
      Exit;
    end;

  Result := -1;
end;

procedure TScDualListEditorForm.lbDestDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := (Source = lbSource) and (lbSource.SelCount > 0) or
    (Source = lbDest) and (lbDest.SelCount = 1);
end;

procedure TScDualListEditorForm.lbDestDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  Pos: TPoint;
  Index, SelIndex: integer;
begin
  Pos.X := X;
  Pos.Y := Y;
  Index := lbDest.ItemAtPos(Pos, False);
  if Source = lbSource then
    MoveSelected(lbSource, lbDest, Index, False)
  else
  if Source = lbDest then begin
    SelIndex := GetFirstSelection(lbDest);
    Move(lbDest, SelIndex, Index);
  end;
end;

procedure TScDualListEditorForm.lbSourceDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := (Source = lbDest) and (lbDest.SelCount > 0);
end;

procedure TScDualListEditorForm.lbSourceDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  Pos: TPoint;
  Index: integer;
begin
  if Source = lbDest then begin
    Pos.X := X;
    Pos.Y := Y;
    Index := lbSource.ItemAtPos(Pos, False);
    MoveSelected(lbDest, lbSource, Index, False);
  end;
end;

procedure TScDualListEditorForm.Move(List: TListBox; FromIndex, ToIndex: integer);
begin
  if (FromIndex >= 0) and (ToIndex >= 0) and (ToIndex < List.Items.Count) and (FromIndex <> ToIndex)
  then begin
    List.Items.Move(FromIndex, ToIndex);
    List.Selected[ToIndex] := True;
    FModified := True;
    SetButtons;
  end;
end;

procedure TScDualListEditorForm.btnUpClick(Sender: TObject);
var
  SelIndex: integer;
begin
  SelIndex := GetFirstSelection(lbDest);
  Move(lbDest, SelIndex, SelIndex - 1);
end;

procedure TScDualListEditorForm.btnDownClick(Sender: TObject);
var
  SelIndex: integer;
begin
  SelIndex := GetFirstSelection(lbDest);
  Move(lbDest, SelIndex, SelIndex + 1);
end;

procedure TScDualListEditorForm.lbDestKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Shift = [ssCtrl] then
    case Key of
      VK_UP: begin
        btnUpClick(nil);
        Key := 0;
      end;
      VK_DOWN: begin
        btnDownClick(nil);
        Key := 0;
      end;
    end;
end;

procedure TScDualListEditorForm.ListClick(Sender: TObject);
begin
  SetButtons;
end;

procedure TScDualListEditorForm.FormShow(Sender: TObject);
var
  List: TStringList;
  i, j: integer;
begin
  FModified := False;

  List := TStringList.Create;
  try
    GetSrcListItems(List);
    AssignStrings(List, lbSource.Items);
    GetDstListItems(List);
    AssignStrings(List, lbDest.Items);
  finally
    List.Free;
  end;

  for i := 0 to lbDest.Items.Count - 1 do begin
    j := lbSource.Items.IndexOf(lbDest.Items[i]);
    if j <> -1 then
      lbSource.Items.Delete(j);
  end;

  SetButtons;
  FModified := False;
  FormResize(nil);
end;

procedure TScDualListEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i: integer;
begin
  if ModalResult <> mrOk then
    Exit;

  FSSLCipherSuites.BeginUpdate;
  try
    FSSLCipherSuites.Clear;

    for i := 0 to lbDest.Count - 1 do
      (FSSLCipherSuites.Add as TScSSLCipherSuiteItem).AsString := lbDest.Items[i];
  finally
    FSSLCipherSuites.EndUpdate;
  end;
end;

procedure TScDualListEditorForm.FormResize(Sender: TObject);
begin
  pnSource.Width := (ClientWidth - pnCenter.Width * 2) div 2;
end;

procedure TScDualListEditorForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then begin
    Close;
    Key := #0;
  end;
end;

end.
