
//////////////////////////////////////////////////
//  SecureBridge Components
//  Copyright  2007-2024 Devart. All right reserved.
//  Base Component Editor
//////////////////////////////////////////////////

{$I SB.inc}

unit ScEditor;

interface
uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, Buttons,
{$IFDEF FPC}
  LResources, LCLType,
{$ENDIF}
  ScTypes, ScDesignUtils;

type
  TScEditorClass = class of TScEditorForm;

  TScEditorForm = class(TForm)
    BtnPanel: TPanel;
    btOk: TBitBtn;
    btCancel: TBitBtn;
    imCorner: TImage;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure CloseClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
  {$IFDEF MSWINDOWS}
    FOldPosition: TPosition;
  {$ENDIF}

    FConfirmCancel: boolean;

    procedure SetConfirmCancel(const Value: boolean);
  protected
    FModified: boolean;
    FolderName: string;
    FScDesignUtilsClass: TScDesignUtilsClass;
    FInitialProperty: string;

    function GetModified: boolean; virtual;
    procedure SetModified(Value: boolean); virtual;

    procedure Resize; override;

    procedure DoInit; virtual;
    procedure DoActivate; virtual;
    procedure DoSave; virtual;

    procedure SaveControlData; virtual;

    function SaveState: boolean; virtual;
    function LoadState: boolean; virtual;

    function GetComponent: TComponent; virtual;
    procedure SetComponent(Value: TComponent); virtual;

  {$IFDEF MSWINDOWS}
    function KeyPath: string;
  {$ENDIF}
    property Modified: boolean read GetModified write SetModified;

  public
    constructor Create(Owner: TComponent; ScDesignUtilsClass: TScDesignUtilsClass); reintroduce; virtual;

  /// If ConfirmCancel is True editor asks user for comfirmation to discard
  /// changes. By default ConfirmCancel is set to True.
    property ConfirmCancel: boolean read FConfirmCancel write SetConfirmCancel;

    property ScDesignUtilsClass: TScDesignUtilsClass read FScDesignUtilsClass;

    property Component: TComponent read GetComponent write SetComponent;
    property InitialProperty: string read FInitialProperty write FInitialProperty;
  end;

implementation
uses
{$IFDEF MSWINDOWS}
  Registry,
{$ENDIF}
  TypInfo;

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

{ TScEditorForm }

constructor TScEditorForm.Create(Owner: TComponent; ScDesignUtilsClass: TScDesignUtilsClass);
begin
  inherited Create(Owner);
  FScDesignUtilsClass := ScDesignUtilsClass;
  FolderName := ClassName;
  ConfirmCancel := True;
end;

procedure TScEditorForm.DoInit;
begin
  Modified := False;

  Assert(Component <> nil, ClassName);
  if Component.Owner <> nil then
    Caption := Component.Owner.Name + '.' + Component.Name
  else
    Caption := Component.Name;

  Resize;
end;

procedure TScEditorForm.DoActivate;
begin
end;

procedure TScEditorForm.DoSave;
begin
  SaveControlData;
end;

procedure TScEditorForm.SaveControlData;
begin
end;

{$IFDEF MSWINDOWS}

function TScEditorForm.SaveState: boolean;
var
  Registry: TRegistry;
begin
  if (BorderStyle <> bsDialog) or (FOldPosition <> poScreenCenter)
  then begin
    Registry := TRegistry.Create(KEY_READ OR KEY_WRITE);
    try
      with Registry do begin
        OpenKey(KeyPath + '\' + FolderName, True);
        if BorderStyle in [bsSizeable, bsSizeToolWin] then begin
          WriteBool('Maximized', WindowState = wsMaximized);
          WriteBool('Minimized', WindowState = wsMinimized);
          if WindowState = wsNormal then begin
            WriteInteger('Width', Width);
            WriteInteger('Height', Height);
          end;
        end;
        if (FOldPosition <> poScreenCenter) and (WindowState = wsNormal) then begin
          WriteInteger('Left', Left);
          WriteInteger('Top', Top);
        end;
        {if not Modal then
          SetBool('Visible', Visible);}
        Result := True;
      end
    finally
      Registry.Free;
    end
  end
  else
    Result := False;
end;

function TScEditorForm.LoadState: boolean;
var
  Registry: TRegistry;
  _left, _top: integer;
begin
  Registry := TRegistry.Create(KEY_READ OR KEY_WRITE);
  try
    with Registry do begin
      if OpenKey(KeyPath + '\' + FolderName, False) then begin
        if Position <> poScreenCenter then begin
          if ValueExists('Left') then begin
            _left := ReadInteger('Left');
            if (_left < 0) or (_left >= Screen.DesktopWidth) then
              _left := (Screen.Width - Width) div 2;
            Left := _left;
          end;
          if ValueExists('Top') then begin
            _top := ReadInteger('Top');
            if (_top < 0) or (_top >= Screen.DesktopHeight) then
              _top := (Screen.Height - Height) div 2;
            Top := _top;
          end;
        end;
        if BorderStyle in [bsSizeable, bsSizeToolWin] then begin
          if ValueExists('Width') then
            Width := ReadInteger('Width');
          if ValueExists('Height') then
            Height := ReadInteger('Height');

          if ValueExists('Maximized') and ReadBool('Maximized') then
            WindowState := wsMaximized
          else
            {if ValueExists('Minimized') and ReadBool('Minimized') then
              WindowState := wsMinimized
            else}
            WindowState := wsNormal;
        end;
        if ValueExists('Visible') and ReadBool('Visible') then begin
          Show;
          Update;
        end;
        Result := True;
      end
      else
      begin
        Left := (Screen.Width - Width) div 2;
        Top := (Screen.Height - Height) div 2;
        Result := False;
      end;
    end;
  finally
    Registry.Free;
  end;
  FOldPosition := Position;
end;

function TScEditorForm.KeyPath: string;
begin
  Result := '\SOFTWARE\Devart\' + FScDesignUtilsClass.GetProjectName + '\Editors';
end;

{$ENDIF}

{$IFDEF UNIX}
function TScEditorForm.SaveState: boolean;
begin
  Result := True;
end;

function TScEditorForm.LoadState: boolean;
begin
  Position := poScreenCenter;
  Result := True;
end;
{$ENDIF}

function TScEditorForm.GetComponent: TComponent;
begin
  Assert(False, 'Must be overriden');
  Result := nil;
end;

procedure TScEditorForm.SetComponent(Value: TComponent);
begin
  Assert(False, 'Must be overriden');
end;

procedure TScEditorForm.FormShow(Sender: TObject);
begin
  LoadState;

{$IFNDEF MSWINDOWS}
  DoInit;
{$ELSE}
  try
    DoInit;
  except
    on E: EAbort do begin
      PostMessage(Handle, WM_CLOSE, 0, 0);
    end;
  end;
{$ENDIF}
  DoActivate;
end;

procedure TScEditorForm.FormHide(Sender: TObject);
begin
  SaveState;
end;

procedure TScEditorForm.SaveClick(Sender: TObject);
begin
  DoSave;

  Modified := False;

  ModalResult := mrOk;
end;

procedure TScEditorForm.CloseClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TScEditorForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
const
  ResYes = IDYES;
  ResNo = IDNO;
var
  Res: integer;
begin
  try
    SaveControlData;
  except
    // Silent. User can enter wrong data
  end;

  if Modified then begin
    if ConfirmCancel then
      Res := Application.MessageBox('Save changes to component?', 'Confirm',
        MB_YESNOCANCEL or MB_ICONQUESTION)
    else
      Res := ResNo;

    if Res = ResYes then begin
      if ModalResult <> mrYesToAll then    // to let show the IDE Editor
        ModalResult := mrOk;
      DoSave;
      Modified := False;
      CanClose := True;
    end
    else
      if Res = ResNo then begin
        CanClose := True;
        if ModalResult <> mrYesToAll then  // to let show the IDE Editor
          ModalResult := mrCancel;
      end
      else
        CanClose := False;
  end
  else
    CanClose := True;
end;

procedure TScEditorForm.SetConfirmCancel(const Value: boolean);
begin
  FConfirmCancel := Value;
end;

procedure TScEditorForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then begin
    Close;
    Key := #0;
  end;
end;

function TScEditorForm.GetModified: boolean;
begin
  Result := FModified;
end;

procedure TScEditorForm.SetModified(Value: boolean);
begin
  FModified := Value;
end;

procedure TScEditorForm.Resize;
begin
  inherited;

  if imCorner <> nil then begin
    imCorner.Top := BtnPanel.Height - imCorner.Height;
    imCorner.Left := BtnPanel.Width - imCorner.Width;

    btCancel.Top := 8;
    btCancel.Left := BtnPanel.Width - btCancel.Width - 8;

    btOK.Top := 8;
    btOK.Left := BtnPanel.Width - btCancel.Width - 6 - btOK.Width - 8;
  end;
end;

end.
